@@ -56,7 +56,7 @@ use Carp qw/croak carp confess cluck/;
 use Scalar::Util qw/looks_like_number blessed reftype/;
 use Unicode::UTF8 qw/decode_utf8 valid_utf8/;
 use B;
-our $VERSION = '0.27_01';
+our $VERSION = '0.28_01';
 
 # http://stackoverflow.com/questions/1185822/how-do-i-create-or-test-for-nan-or-infinity-in-perl#1185828
 
@@ -328,7 +328,13 @@ sub create_json_recursively
     if ($ref) {
 	if ($ref eq 'HASH') {
 	    $jc->openB ('{');
-	    for my $k (keys %$input) {
+	    my @keys = keys %$input;
+	    if ($jc->{_sort}) {
+		@keys = sort @keys;
+	    }
+	    my $i = 0;
+	    my $n = scalar (@keys);
+	    for my $k (@keys) {
 		my $error = stringify ($jc, $k);
 		if ($error) {
 		    return $error;
@@ -344,13 +350,17 @@ sub create_json_recursively
 			return $error;
 		    }
 		}
-		$jc->comma ();
+		$i++;
+		if ($i < $n) {
+		    $jc->comma ();
+		}
 	    }
-	    $jc->{output} =~ s/,(\s*)$/$1/;
 	    $jc->closeB ('}');
 	}
 	elsif ($ref eq 'ARRAY') {
 	    $jc->openB ('[');
+	    my $i = 0;
+	    my $n = scalar (@$input);
 	    for my $k (@$input) {
 		my $bool = isbool (\$k);
 		if ($bool) {
@@ -362,9 +372,11 @@ sub create_json_recursively
 			return $error;
 		    }
 		}
-		$jc->comma ();
+		$i++;
+		if ($i < $n) {
+		    $jc->comma ();
+		}
 	    }
-	    $jc->{output} =~ s/,(\s*)$/$1/;
 	    $jc->closeB (']');
 	}
 	elsif ($ref eq 'SCALAR') {
@@ -611,4 +623,10 @@ sub indent
     $jc->{_indent} = !! $onoff;
 }
 
+sub JSON::Create::PP::sort
+{
+    my ($jc, $onoff) = @_;
+    $jc->{_sort} = !! $onoff;
+}
+
 1;
